
/*

	Open Kray Instance Plugin
	Matt Gorner
	05.11.2009
	
	v1.1
	
	Script to open the Kray Instance plugin on the selected object from the main UI without having to open the Object Properties window
	
	NOTE: This script will only add ONE instance of the plugin each time is it run, despite the fact that Kray allows more per object.
	If the object has MORE than one instance of the plugin (i.e. the user has added more themselves).  Then the script will bring up the options for the FIRST plugin in the list.

*/


@version 2.3
@script generic
@name Kray Open Instance

generic
{
	selected_item = Scene().getSelect();

	/*
	genus 1 = Mesh
	genus 2 = Light
	genus 3 = Camera
	genus 4 = Bone
	*/

	/* Check type ... */
	if(selected_item){
		item_type = selected_item[1].genus;
	}else
	{
		error("Kray Message: Nothing is selected");
	}

	if(item_type != 1)
		error("Kray Message: You must select an object to use Kray Instances");

	/* Set the break variable */
    appActive = false;

	/* Store a list of Custom Object Handlers attached to the selected object */
    object_handlers = Scene().firstSelect().server( "CustomObjHandler" );

	/* If a list exists ... */
    if(object_handlers)
    {
    	/* Get the size of the array */
        counter = size(object_handlers);

		/* Loop through until we find the 'KrayInstances' plugin */
        for(x = 1; x <= counter; x++){
            if(object_handlers[x] == "KrayInstances"){
                appActive = true;
                break;
            }
        }
    }

	/* If it wasn't found, add the 'KrayInstances' plugin */
	if(!appActive){
        ApplyServer("CustomObjHandler","KrayInstances");
	}

	/* Reset break variable */
    appActive = false;

	/* Refresh the list of Custom Object Handlers attached to the selected object, otherwise the one we just added won't be there */
    object_handlers = Scene().firstSelect().server( "CustomObjHandler" );

	/* If a list exists ... */
    if(object_handlers){

    	/* Get the size of the array */
        counter = size(object_handlers);

		/* Loop through until we find the 'KrayInstances' plugin */
        for(x = 1; x <= counter; x++){
            if(object_handlers[x] == "KrayInstances"){
                appActive = true;
                break;
            }
        }
    }

	/* If it WAS found, EDIT the 'KrayInstances' plugin instead */
	if (appActive){

		/* Bring up the plugin interface */
		EditServer("CustomObjHandler",x);

	}else{

		/* Something went wrong, tell the user */
		error("Kray Message: An error occured trying to edit the Kray Instances Plugin");
	}
}
