
/*

	Open Kray Photon Multiplier Plugin
	Matt Gorner
	05.11.2009
	
	v1.1
	
	Script to open the Kray Photon Multiplier plugin on the selected light from the main UI without having to open the Motion Options window

*/


@version 2.3
@script generic
@name Kray Open Photon Multiplier

generic
{

	/* Get items selected ... */
	selected_item = Scene().getSelect();

	/* Check type ... */
	if(selected_item){
		item_type = selected_item[1].genus;
	}else
	{
		error("Kray Message: Nothing is selected");
	}

	/*
	genus 1 = Mesh
	genus 2 = Light
	genus 3 = Camera
	genus 4 = Bone
	*/

	if( item_type == 1 && item_type == 2 || item_type == 3 || item_type == 4)
		error("Kray Message: You must select a light or object to use Kray Photon Multiplier");

	/* Set the break variable */
    appActive = false;

	/* Store a list of Motion Handlers attached to the selected object */
    motion_handlers = Scene().firstSelect().server( "ItemMotionHandler" );

	/* If a list exists ... */
    if(motion_handlers)
    {
    	/* Get the size of the array */
        counter = size(motion_handlers);

		/* Loop through until we find the 'KrayPhotonMultiplier' plugin */
        for(x = 1; x <= counter; x++){
            if(motion_handlers[x] == "KrayPhotonMultiplier"){
                appActive = true;
                break;
            }
        }
    }

	/* If it wasn't found, add the 'KrayPhotonMultiplier' plugin */
	if(!appActive){
        ApplyServer("ItemMotionHandler","KrayPhotonMultiplier");
	}

	/* Reset break variable */
    appActive = false;

	/* Refresh the list of Motion Handlers attached to the selected object, otherwise the one we just added won't be there */
    motion_handlers = Scene().firstSelect().server( "ItemMotionHandler" );

	/* If a list exists ... */
    if(motion_handlers){

    	/* Get the size of the array */
        counter = size(motion_handlers);

		/* Loop through until we find the 'KrayPhotonMultiplier' plugin */
        for(x = 1; x <= counter; x++){
            if(motion_handlers[x] == "KrayPhotonMultiplier"){
                appActive = true;
                break;
            }
        }
    }

	/* If it WAS found, EDIT the 'KrayPhotonMultiplier' plugin instead */
	if (appActive){

		/* Bring up the plugin interface */
		EditServer("ItemMotionHandler",x);

	}else{

		/* Something went wrong, tell the user */
		error("Kray Message: An error occured trying to edit the Kray Photon Multiplier Plugin");
	}
}
